-- ==========================================================================================
-- Ada Distilled              by Richard Riehle, AdaWorks Software Engineering
--                                               For your Ada training and consulting needs
-- Block_Example_1.adb
--
-- See 4.2.1 in Ada Distilled for discussion of the case statements.
--  
-- Line numbers correspond to those in Ada Distilled.
-- 
-- ==========================================================================================
--
with Ada.Text_Io,	          --   Note the comma instead of semicolon
Ada.Integer_Text_Io; 
procedure Block_Example_1 is
  
   Outcome : Integer;

   -- with Ada.Text_Io,	                                  --  1 Note the comma instead of semicolon
   -- Ada.Integer_Text_Io; 	                            --  2 Predefined package for Integer I/O
   function Get return Integer  is	                      --  3 Parameterless function
      package Iio renames Ada.Integer_Text_Io;            --  4 Make the name shorter via renames clause
      package Tio renames Ada.Text_Io;	                --  5 Make the name shorter
      Data : Integer := -0;	                            --  6 In scope for all of  P8
      Try_Limit : constant := 3; 	                      --  7 A constant cannot be changed
      Try_Count : Natural := 0;	                      --  8 Natural cannot be less than zero
      begin				                            --  9 Required to initiated sequence of statements
      Input_Loop:		                                  -- 10 Optional label for the loop
         loop			                            -- 11 Required reserved word
         Try_Block:	                                  -- 12 Always name a begin..end block
            begin		                                  -- 13 Start begin ... end block
            Try_Count := Try_Count + 1;	                -- 14 Increment a variable by one
            Iio.Get(Data);	                            -- 15 Convert external text to internal number
               exit Input_Loop;	                      -- 16 unconditional loop exit
         exception	                                  -- 17 Placed between begin ... end sequence
            when Tio.Data_Error =>	                      -- 18 Exception handling 
               if Try_Count > Try_Limit then              -- 19 Decide whether to exit the loop
                  Ada.Text_Io.Put_Line("Too many tries"); -- 20 Because the Try_Count is too high
                     exit Input_Loop;	                -- 21 exit the loop
                  end if;	                            -- 22 Every if requires an end if
            end Try_Block;                                -- 23 The label is required if block is labeled
         end loop Input_Loop;	                            -- 24 Loop is labeled so label is required
         return  Data;	                                  -- 25  One return statement for this function
      end Get;			                            -- 26 Always label a subprogram end statement
   begin
       Outcome := Get;
   end Block_Example_1;