-- ==========================================================================================
-- Ada Distilled              by Richard Riehle, AdaWorks Software Engineering
--                                               For your Ada training and consulting needs
-- Main.adb
--
-- See 6.1.4 in Ada Distilled for discussion of this program.
--
-- Demonstrates visiblity issues for infix operators
--
-- Line numbers correspond to those in Ada Distilled.
-- 
-- =========================================================================================
-- 
-- ==================================================	            --  1  
-- diamond.ada		                                          --  2  
-- Solution to Diamond Problem by LCPL Mathiowetz, USMC	      --  3  
-- Camp Kinser, Okinawa.  June 1993.  	                        --  4  
-- ==================================================	            --  5 .
with Ada.Text_Io; use Ada; -- Makes all of package Ada visible	--  6 Only Text_IO is required for this program
procedure Diamond is	                                          --  7  Specification with no parameters
   package Tio renames Text_Io;	                              --  8  A shortened name for Text_IO
   subtype Column is Tio.Positive_Count;                          --  9  Subtype may be used with its parent type
   Center : constant := 37;	                                    -- 10 A named constant
   Left_Temp, Right_Temp : Integer := Center;                     -- 11 Temporary values, initialized
   Plus_2     : constant :=  2;	                              -- 12 Positve constant value
   Minus_2 : constant := -2;	                                    -- 13 Negative constant value
   procedure Draw (Left, Right, Depth : in Integer) is            -- 14 Nested procedure with parameter list
      Symbol : String(1..1) := "X";	                              -- 15 The character we will print
      Left_Col, Right_Col : Column;	                              -- 16 These are probably extraneous
   begin			                                          -- 17 We are in a nested procedure
      for Index in 1..Depth loop                                  -- 18  Index declared here; type is range type
         if Left_Temp = Center then                               -- 19 Is it time to Put the center character?
            Tio.Set_Col(Center);	                              -- 20 Using renamed Text_IO.Count
            Tio.Put(Symbol);	                                    -- 21 
         else			                                    -- 22 
            Left_Col  := Column(Left_Temp);                       -- 23 Extraneous assignment on these two lines;
            Right_Col := Column(Right_Temp);                      -- 24 we could do type conversion in Set_Col
            Tio.Set_Col(Left_Col);	                              -- 25 TIO.Set_Col(Column(Right_Temp))
            Tio.Put(Symbol);                                      -- 26 might be better coding on line 25 and 27
            Tio.Set_Col(Right_Col);	                              -- 27 
            Tio.Put(Symbol);	                                    -- 28 Symbol on line 15
         end if;		                                          -- 29 
         Tio.New_Line;	                                          -- 30 
         Left_Temp  := Left_Temp  + Left;                         -- 31 Arithmetic on Temporary values using
         Right_Temp := Right_Temp + Right;                        -- 32 algebraic addition on negative parameter
      end loop;		                                          -- 33 
   end Draw;		                                          -- 34  End of nested procedure
begin -- Diamond	                                                -- 35  Always comment this kind of thing
   Draw (Left => Minus_2, Right => Plus_2,  Depth =>  9);         -- 36 Use named association for these calls.
   Draw (Left => Plus_2,  Right => Minus_2, Depth => 10);         -- 37 Reverse the signs to get a different shape
end Diamond;		                                          -- 38 End of unit with named unit at end