package Record_Declarations is                   --  1 This specification would require a pragma Elaborate_Body
   type Library_Book is                          --  2 Simple constrained record
      record                                     --  3 reserved word, record
         Isbn             : String (1 .. 12);    --  4 String component 
         Title            : String (1 .. 30);    --  5 String component 
         Author           : String (1 .. 40);    --  6 String component 
         Purchase_Price   : Float;               --  7 Floating point component 
         Copies_Available : Natural;             --  8 Subtype natural from package Standard 
      end record;  	                         --  9 Must identify end of scope of each record
                                                 -- 10 
   type Message_1 is                             -- 11 Simple record with an
      record                                     -- 12                 unconstrained data type
         Text   : Unbounded_String;              -- 13 See ALRM A.4.5 
         Length : Natural;                       -- 14 See package Standard 
      end record;  	                         -- 15 
                                                 -- 16
   type Message_2 (Size : Positive) is           -- 17 Record with a discriminant
      record                                     -- 18 This must be constrained before
         Text   : String (1 .. Size);            -- 19 it may be used.  Note that the Size 
         Length : Natural;                       -- 20 has a corresponding entry in the record 
      end record;  	                         -- 21 Dynamically allocated records might not
                                                 -- 22 be as efficient as you would like.
   type Message_3 (Size : Positive := 1) is      -- 23 Record with a default discriminant
      record                                     -- 24 This may be constrained or may use
         Text   : String (1 .. Size);            -- 25 the default constraint.  There are more 
         Length : Natural;                       -- 26 rules for this, but we defer them to an 
      end record;  	                         -- 27 advancd discussion of the language
                                                 -- 28
   type Message_4 is tagged                      -- 29 A tagged type.  This may be extended.
      record                                     -- 30 with more components
         Text   : Unbounded_String;              -- 31 Unbounded String(See Ada.Fixed.Unbounded). 
         Length : Natural;                       -- 32  
      end record;  	                         -- 33 
   .				                         -- 34 
   type Message_5 is new Message_4 with          -- 35 Derived from a tagged type and one 
      record                                     -- 36 additional component.  This record now x
         Stamp : Calendar.Time                   -- 37 has a total of three components, those       
      end record;  	                         -- 38 it inherits and the one defined within it.
                                                 -- 39 

   type Message_6 is                             -- 40 Record containing another record 
      record                                     -- 41  
         Message_Data : Message_1;               -- 42 See line 11 
         Library_Data : Library_Book;            -- 43 See line 2 
      end record;  	                         -- 44.
end Record_Declarations;                         -- 45 This package might require a pragma Elaborate_Body